@@ -1,6 +1,4 @@
-use clap::builder::StyledStr;
-
-use crate::utils::hyphens_to_underscores;
+use crate::utils::{escape_shell_words, hyphens_to_underscores};
 
 pub const VARIABLE_PREFIX: &str = "argc";
 
@@ -12,14 +10,17 @@ pub enum ArgcValue {
     PositionalSingle(String, String),
     PositionalSingleFn(String, String),
     PositionalMultiple(String, Vec<String>),
+    ExtraPositionalMultiple(Vec<String>),
     CmdFn(String),
     ParamFn(String),
-    ClapError((StyledStr, i32)),
+    Error((String, i32)),
 }
 
 impl ArgcValue {
-    pub fn to_shell(values: Vec<Self>, no_color: bool) -> String {
+    pub fn to_shell(values: Vec<Self>) -> String {
         let mut variables = vec![];
+        let mut last = String::new();
+        let mut call = String::new();
         let mut positional_args = vec![];
         for value in values {
             match value {
@@ -28,7 +29,7 @@ impl ArgcValue {
                         "{}_{}={}",
                         VARIABLE_PREFIX,
                         hyphens_to_underscores(&name),
-                        value
+                        escape_shell_words(&value)
                     ));
                 }
                 ArgcValue::SingleFn(name, fn_name) => {
@@ -44,17 +45,22 @@ impl ArgcValue {
                         "{}_{}=( {} )",
                         VARIABLE_PREFIX,
                         name,
-                        values.join(" ")
+                        values
+                            .iter()
+                            .map(|v| escape_shell_words(v))
+                            .collect::<Vec<String>>()
+                            .join(" ")
                     ));
                 }
                 ArgcValue::PositionalSingle(name, value) => {
+                    let value = escape_shell_words(&value);
                     variables.push(format!(
                         "{}_{}={}",
                         VARIABLE_PREFIX,
                         hyphens_to_underscores(&name),
                         &value
                     ));
-                    positional_args.push(value.to_string());
+                    positional_args.push(value);
                 }
                 ArgcValue::PositionalSingleFn(name, fn_name) => {
                     variables.push(format!(
@@ -66,6 +72,10 @@ impl ArgcValue {
                     positional_args.push(format!("`{}`", fn_name));
                 }
                 ArgcValue::PositionalMultiple(name, values) => {
+                    let values = values
+                        .iter()
+                        .map(|v| escape_shell_words(v))
+                        .collect::<Vec<String>>();
                     variables.push(format!(
                         "{}_{}=( {} )",
                         VARIABLE_PREFIX,
@@ -74,31 +84,49 @@ impl ArgcValue {
                     ));
                     positional_args.extend(values);
                 }
+                ArgcValue::ExtraPositionalMultiple(values) => {
+                    let values = values
+                        .iter()
+                        .map(|v| escape_shell_words(v))
+                        .collect::<Vec<String>>();
+                    positional_args.extend(values);
+                }
                 ArgcValue::CmdFn(name) => {
                     if positional_args.is_empty() {
-                        variables.push(name.to_string());
+                        last = name.to_string();
                     } else {
-                        variables.push(format!("{} {}", name, positional_args.join(" ")));
+                        last = format!("{} {}", name, positional_args.join(" "));
                     }
+                    call = name.clone();
                 }
                 ArgcValue::ParamFn(name) => {
                     if positional_args.is_empty() {
-                        variables.push(format!("{name};exit;"));
+                        last = format!("{name};exit;");
                     } else {
-                        variables.push(format!("{} {};exit;", name, positional_args.join(" ")));
+                        last = format!("{} {};exit;", name, positional_args.join(" "));
                     }
+                    call = name.clone();
                 }
-                ArgcValue::ClapError((error, exit)) => {
-                    variables.clear();
-                    let error = if no_color {
-                        error.to_string()
-                    } else {
-                        error.ansi().to_string()
-                    };
-                    variables.push(format!("cat >&2 <<-'EOF' \n{}\nEOF\nexit {}", error, exit));
+                ArgcValue::Error((error, exit)) => {
+                    return format!("cat >&2 <<-'EOF' \n{}\nEOF\nexit {}", error, exit)
                 }
             }
         }
+
+        variables.push(format!(
+            "{}__args=( {} )",
+            VARIABLE_PREFIX,
+            positional_args.join(" ")
+        ));
+
+        if !call.is_empty() {
+            variables.push(format!("{}__fn={}", VARIABLE_PREFIX, call));
+        }
+
+        if !last.is_empty() {
+            variables.push(last);
+        }
+
         variables.join("\n")
     }
 
