@@ -1,7 +1,8 @@
-use crate::param::{FlagParam, OptionParam, ParamData, PositionalParam};
+use crate::param::{FlagOptionParam, ParamData, PositionalParam};
 use crate::utils::{is_choice_value_terminate, is_default_value_terminate};
 use crate::Result;
 use anyhow::bail;
+use nom::character::complete::one_of;
 use nom::{
     branch::alt,
     bytes::complete::{escaped, tag, take_till, take_while1},
@@ -9,7 +10,7 @@ use nom::{
         complete::{anychar, char, satisfy, space0, space1},
         streaming::none_of,
     },
-    combinator::{eof, fail, map, opt, peek, rest, success},
+    combinator::{eof, fail, map, not, opt, peek, rest, success},
     multi::{many0, many1, separated_list1},
     sequence::{delimited, pair, preceded, separated_pair, terminated, tuple},
 };
@@ -33,12 +34,10 @@ pub enum EventData {
     Cmd(String),
     /// Define alias for a subcommand, e.g. `@alias t,tst`
     Aliases(Vec<String>),
-    /// Define a option parameter
-    Option(OptionParam),
+    /// Define a flag or option parameter
+    FlagOption(FlagOptionParam),
     /// Define a positional parameter
     Positional(PositionalParam),
-    /// Define a flag
-    Flag(FlagParam),
     /// A shell function. e.g `function cmd()` or `cmd()`
     Func(String),
     /// Placeholder for unknown or invalid tag
@@ -61,12 +60,36 @@ impl Default for EventScope {
 /// Tokenize shell script
 pub fn parse(source: &str) -> Result<Vec<Event>> {
     let mut result = vec![];
-    for (line_idx, line) in source.lines().enumerate() {
+    let lines: Vec<&str> = source.lines().collect();
+    let mut line_idx = 0;
+    while line_idx < lines.len() {
+        let line = lines[line_idx];
         let position = line_idx + 1;
         match parse_line(line) {
             Ok((_, maybe_token)) => {
                 if let Some(maybe_data) = maybe_token {
                     if let Some(data) = maybe_data {
+                        let data = match data {
+                            EventData::Describe(mut text) => {
+                                line_idx += take_comment_lines(&lines, line_idx + 1, &mut text);
+                                EventData::Describe(text)
+                            }
+                            EventData::Cmd(mut text) => {
+                                line_idx += take_comment_lines(&lines, line_idx + 1, &mut text);
+                                EventData::Cmd(text)
+                            }
+                            EventData::FlagOption(mut param) => {
+                                line_idx +=
+                                    take_comment_lines(&lines, line_idx + 1, &mut param.describe);
+                                EventData::FlagOption(param)
+                            }
+                            EventData::Positional(mut param) => {
+                                line_idx +=
+                                    take_comment_lines(&lines, line_idx + 1, &mut param.describe);
+                                EventData::Positional(param)
+                            }
+                            v => v,
+                        };
                         result.push(Event { position, data });
                     } else {
                         bail!("syntax error at line {}", position)
@@ -77,6 +100,7 @@ pub fn parse(source: &str) -> Result<Vec<Event>> {
                 bail!("fail to parse at line {}, {}", position, err)
             }
         }
+        line_idx += 1;
     }
     Ok(result)
 }
@@ -140,11 +164,11 @@ fn parse_tag_param(input: &str) -> nom::IResult<&str, Option<EventData>> {
     let arg = alt((
         map(
             preceded(pair(tag("flag"), space1), parse_flag_param),
-            |param| Some(EventData::Flag(param)),
+            |param| Some(EventData::FlagOption(param)),
         ),
         map(
             preceded(pair(tag("option"), space1), parse_option_param),
-            |param| Some(EventData::Option(param)),
+            |param| Some(EventData::FlagOption(param)),
         ),
         map(
             preceded(pair(tag("arg"), space1), parse_positional_param),
@@ -171,37 +195,35 @@ fn parse_tag_unknown(input: &str) -> nom::IResult<&str, Option<EventData>> {
 }
 
 // Parse `@option`
-fn parse_option_param(input: &str) -> nom::IResult<&str, OptionParam> {
+fn parse_option_param(input: &str) -> nom::IResult<&str, FlagOptionParam> {
     alt((parse_with_long_option_param, parse_no_long_option_param))(input)
 }
 
 // Parse `@option` with long name
-fn parse_with_long_option_param(input: &str) -> nom::IResult<&str, OptionParam> {
+fn parse_with_long_option_param(input: &str) -> nom::IResult<&str, FlagOptionParam> {
     map(
         tuple((
             parse_short,
-            preceded(
-                pair(space0, tag("--")),
-                alt((
-                    parse_param_modifer_choices_default,
-                    parse_param_modifer_choices_fn,
-                    parse_param_modifer_choices,
-                    parse_param_assign_fn,
-                    parse_param_assign,
-                    parse_param_modifer,
-                )),
-            ),
+            preceded(space0, alt((tag("--"), tag("-")))),
+            alt((
+                parse_param_modifer_choices_default,
+                parse_param_modifer_choices_fn,
+                parse_param_modifer_choices,
+                parse_param_assign_fn,
+                parse_param_assign,
+                parse_param_modifer,
+            )),
             parse_zero_or_many_value_notations,
             parse_tail,
         )),
-        |(short, arg, value_names, summary)| {
-            OptionParam::new(arg, summary, short, false, &value_names)
+        |(short, dashes, arg, value_names, describe)| {
+            FlagOptionParam::new(arg, describe, short, false, dashes, &value_names)
         },
     )(input)
 }
 
 // Parse `@option` without long name
-fn parse_no_long_option_param(input: &str) -> nom::IResult<&str, OptionParam> {
+fn parse_no_long_option_param(input: &str) -> nom::IResult<&str, FlagOptionParam> {
     map(
         tuple((
             preceded(
@@ -221,9 +243,9 @@ fn parse_no_long_option_param(input: &str) -> nom::IResult<&str, OptionParam> {
             parse_zero_or_many_value_notations,
             parse_tail,
         )),
-        |(arg, value_names, summary)| {
+        |(arg, value_names, describe)| {
             let short = arg.name.chars().next();
-            OptionParam::new(arg, summary, short, true, &value_names)
+            FlagOptionParam::new(arg, describe, short, false, "", &value_names)
         },
     )(input)
 }
@@ -243,36 +265,39 @@ fn parse_positional_param(input: &str) -> nom::IResult<&str, PositionalParam> {
             parse_zero_or_one_value_notation,
             parse_tail,
         )),
-        |(arg, value_name, summary)| PositionalParam::new(arg, summary, value_name),
+        |(arg, value_name, describe)| PositionalParam::new(arg, describe, value_name),
     )(input)
 }
 
 // Parse `@flag`
-fn parse_flag_param(input: &str) -> nom::IResult<&str, FlagParam> {
+fn parse_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam> {
     alt((parse_with_long_flag_param, parse_no_long_flag_param))(input)
 }
 // Parse `@flag`
-fn parse_with_long_flag_param(input: &str) -> nom::IResult<&str, FlagParam> {
+fn parse_with_long_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam> {
     map(
         tuple((
             parse_short,
-            preceded(pair(space0, tag("--")), parse_long_flag_and_asterisk),
+            preceded(space0, alt((tag("--"), tag("-")))),
+            parse_long_flag_and_asterisk,
             parse_tail,
         )),
-        |(short, arg, summary)| FlagParam::new(arg, summary, short, false),
+        |(short, dashes, arg, describe)| {
+            FlagOptionParam::new(arg, describe, short, true, dashes, &[])
+        },
     )(input)
 }
 
 // Parse `@flag` without long name
-fn parse_no_long_flag_param(input: &str) -> nom::IResult<&str, FlagParam> {
+fn parse_no_long_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam> {
     map(
         tuple((
             preceded(pair(space0, tag("-")), parse_short_flag_and_asterisk),
             parse_tail,
         )),
-        |(arg, summary)| {
+        |(arg, describe)| {
             let short = arg.name.chars().next();
-            FlagParam::new(arg, summary, short, true)
+            FlagOptionParam::new(arg, describe, short, true, "", &[])
         },
     )(input)
 }
@@ -375,10 +400,10 @@ fn parse_param_modifer_choices_fn(input: &str) -> nom::IResult<&str, ParamData>
     map(
         pair(
             parse_param_modifer,
-            delimited(char('['), parse_value_fn, char(']')),
+            delimited(char('['), pair(opt(char('?')), parse_value_fn), char(']')),
         ),
-        |(mut arg, choices_fn)| {
-            arg.choices_fn = Some(choices_fn.into());
+        |(mut arg, (validate, choices_fn))| {
+            arg.choices_fn = Some((choices_fn.into(), validate.is_none()));
             arg
         },
     )(input)
@@ -486,6 +511,20 @@ fn parse_notation_text(input: &str) -> nom::IResult<&str, &str> {
     Ok((&input[size - 1..], &input[0..size - 1]))
 }
 
+fn parse_normal_comment(input: &str) -> nom::IResult<&str, &str> {
+    alt((
+        map(tuple((many1(char('#')), space0, eof)), |_| ""),
+        map(
+            tuple((
+                many1(char('#')),
+                opt(one_of(" \t")),
+                not(pair(space0, char('@'))),
+            )),
+            |_| "",
+        ),
+    ))(input)
+}
+
 fn notation_text(input: &str, balances: usize) -> nom::IResult<&str, usize> {
     let (i1, c1) = anychar(input)?;
     match c1 {
@@ -551,6 +590,21 @@ fn is_short_char(c: char) -> bool {
     c.is_ascii() && is_not_fn_name_char(c) && !matches!(c, '-')
 }
 
+fn take_comment_lines(lines: &[&str], idx: usize, output: &mut String) -> usize {
+    let mut count = 0;
+    for line in lines.iter().skip(idx) {
+        if let Ok((text, _)) = parse_normal_comment(line) {
+            output.push('\n');
+            output.push_str(text);
+            count += 1;
+        } else {
+            break;
+        }
+    }
+    *output = output.trim().to_string();
+    count
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -658,6 +712,38 @@ mod tests {
         assert_parse_option_arg!("--foo <<abc def>>");
     }
 
+    #[test]
+    fn test_parse_with_long_option_arg_single_dash() {
+        assert_parse_option_arg!("-f -foo=a <FOO> A foo option");
+        assert_parse_option_arg!("-foo!");
+        assert_parse_option_arg!("-foo+");
+        assert_parse_option_arg!("-foo*");
+        assert_parse_option_arg!("-foo!");
+        assert_parse_option_arg!("-foo=a");
+        assert_parse_option_arg!("-foo=`_foo`");
+        assert_parse_option_arg!("-foo[a|b]");
+        assert_parse_option_arg!("-foo[=a|b]");
+        assert_parse_option_arg!("-foo[`_foo`]");
+        assert_parse_option_arg!("-foo![a|b]");
+        assert_parse_option_arg!("-foo![`_foo`]");
+        assert_parse_option_arg!("-foo![=a|b]", "-foo[=a|b]");
+        assert_parse_option_arg!("-foo+[a|b]");
+        assert_parse_option_arg!("-foo+[`_foo`]");
+        assert_parse_option_arg!("-foo+[=a|b]", "-foo*[=a|b]");
+        assert_parse_option_arg!("-foo*[a|b]");
+        assert_parse_option_arg!("-foo*[=a|b]");
+        assert_parse_option_arg!("-foo*[`_foo`]");
+        assert_parse_option_arg!("-foo <FOO>");
+        assert_parse_option_arg!("-foo-abc <FOO>");
+        assert_parse_option_arg!("-foo=\"a b\"");
+        assert_parse_option_arg!("-foo[\"a|b\"|\"c]d\"]");
+        assert_parse_option_arg!("-foo <abc>");
+        assert_parse_option_arg!("-foo <abc> <def>");
+        assert_parse_option_arg!("-foo <>");
+        assert_parse_option_arg!("-foo <abc def>");
+        assert_parse_option_arg!("-foo <<abc def>>");
+    }
+
     #[test]
     fn test_parse_no_long_option_arg() {
         assert_parse_option_arg!("-f");
@@ -682,6 +768,16 @@ mod tests {
         assert_parse_flag_arg!("--foo*");
     }
 
+    #[test]
+    fn test_parse_with_long_flag_arg_single_dash() {
+        assert_parse_flag_arg!("-f -foo A foo flag");
+        assert_parse_flag_arg!("-. -hidden");
+        assert_parse_flag_arg!("-http1.1");
+        assert_parse_flag_arg!("-foo A foo flag");
+        assert_parse_flag_arg!("-foo");
+        assert_parse_flag_arg!("-foo*");
+    }
+
     #[test]
     fn test_parse_no_long_flag_arg() {
         assert_parse_flag_arg!("-f A foo flag");
@@ -726,8 +822,8 @@ mod tests {
         assert_token!("# @cmd A subcommand", Cmd, "A subcommand");
         assert_token!("# @alias tst", Aliases, vec!["tst"]);
         assert_token!("# @alias t,tst", Aliases, vec!["t", "tst"]);
-        assert_token!("# @flag -f --foo", Flag);
-        assert_token!("# @option -f --foo", Option);
+        assert_token!("# @flag -f --foo", FlagOption);
+        assert_token!("# @option -f --foo", FlagOption);
         assert_token!("# @arg foo", Positional);
         assert_token!("foo()", Func, "foo");
         assert_token!("foo ()", Func, "foo");
@@ -749,7 +845,5 @@ mod tests {
         assert_token!("function foo@bar", Func, "foo@bar");
         assert_token!("foo=bar", Ignore);
         assert_token!("#!/bin/bash", Ignore);
-        assert_token!("# @flag -foo", Error);
-        assert_token!("# @option -foo![=a|b]", Error);
     }
 }
