@@ -17,14 +17,12 @@ pub const SCRIPT_PATHS: [&str; 8] = [
     "dir6/ARGCFILE",
 ];
 
-pub fn get_spec() -> (String, String) {
+pub fn locate_script(script_name: &str) -> String {
     let mut spec_path = std::path::PathBuf::from(env!("CARGO_MANIFEST_DIR"));
     spec_path.push("tests");
-    spec_path.push("spec.sh");
-    (
-        spec_path.display().to_string(),
-        std::fs::read_to_string(spec_path).unwrap(),
-    )
+    spec_path.push("scripts");
+    spec_path.push(script_name);
+    spec_path.display().to_string()
 }
 
 /// Test fixture which creates a temporary directory with a few files and directories inside.
@@ -34,14 +32,19 @@ pub fn get_spec() -> (String, String) {
 pub fn tmpdir() -> TempDir {
     let tmpdir = assert_fs::TempDir::new().expect("Couldn't create a temp dir for tests");
     for path in SCRIPT_PATHS {
-        write_file(&tmpdir, path);
+        let cp = tmpdir_path(&tmpdir, path);
+        if path.ends_with("EMPTY") {
+            cp.write_str("").unwrap();
+        } else {
+            cp.write_str(&get_script(path)).unwrap();
+        }
     }
     tmpdir
 }
 
 #[fixture]
 #[allow(dead_code)]
-pub fn tmpdir2() -> TempDir {
+pub fn tmpdir_bare() -> TempDir {
     assert_fs::TempDir::new().expect("Couldn't create a temp dir for tests")
 }
 
@@ -62,12 +65,25 @@ pub fn tmpdir_path(tmpdir: &TempDir, path: &str) -> ChildPath {
     parts.iter().skip(1).fold(cp, |acc, part| acc.child(part))
 }
 
-fn write_file(tmpdir: &TempDir, path: &str) {
-    let cp = tmpdir_path(tmpdir, path);
-    if path.ends_with("EMPTY") {
-        cp.write_str("").unwrap();
+pub fn create_argc_script(source: &str, name: &str) -> (String, String, assert_fs::NamedTempFile) {
+    let script_content = patch_argc_bin(source);
+    let script_file = assert_fs::NamedTempFile::new(name).unwrap();
+    script_file.write_str(&script_content).unwrap();
+    let script_file = script_file.into_persistent();
+    let script_path = script_file.to_string_lossy().to_string();
+    (script_path, script_content, script_file)
+}
+
+pub fn patch_argc_bin(source: &str) -> String {
+    let argc_path = "./target/debug/argc";
+    if source.contains("--argc-eval") {
+        source.replace("argc --argc-eval", &format!("{argc_path} --argc-eval"))
     } else {
-        cp.write_str(&get_script(path)).unwrap();
+        format!(
+            r###"{source}
+eval "$({argc_path} --argc-eval "$0" "$@")"
+"###,
+        )
     }
 }
 
